/*
 * Tagged Collection document for a Document-View editing architecture
 *
 * Copyright (C) 2003  Enrico Zini <enrico@debian.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * 3B
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "config.h"

#include "DebtagsDocument.h"

#include <tagcoll/InputMerger.h>
#include <tagcoll/ItemGrouper.h>
#include <tagcoll/Expression.h>
#include <tagcoll/StdioParserInput.h>
#include <tagcoll/StringParserInput.h>

#include "Environment.h"

#include <errno.h>

using namespace std;
using namespace Tagcoll;
using namespace aptFront;
using namespace aptFront::cache;
using namespace aptFront::cache::component;

DebtagsDocument::DebtagsDocument() throw () :
	//DebtagsSimple<aptFront::cache::entity::Package>(true),
	_packages(aptFront::cache::Global::get().packages()),
	_debtags(aptFront::cache::Global::get().debtags()),
	_voc(aptFront::cache::Global::get().tags()),
	_changed(false),
	_lastPatch(Tagcoll::Patch<Package, Tag>(Package())),
	_undoTail(0), /*specialsGen(*this),*/ computeIntensive(false)/*, specials(100)*/, hasSpecials(NO)
{
#if 0
	regenerateSpecials();
#endif
}

DebtagsDocument::~DebtagsDocument()
{
#if 0
	MutexLock lock(specialsMutex);
	if (hasSpecials == GENERATING)
	{
		specialsGen.interrupt();
		specialsMutex.unlock();
		specialsGen.join();
		specialsMutex.lock();
	}
#endif
}

#if 0
bool DebtagsDocument::SpecialsGen::advance(int steps) throw ()
{
	fprintf(stderr, "SpecialsGen thread: %d step(s)\n", steps);

	return !_interrupt;
}

template<class ITEM>
class TFLimit : public Tagcoll::Filter<ITEM, Tag>
{
protected:
	int count;

	void consumeItemUntagged(const ITEM& pkg) { if (count > 0) { this->consumer->consume(pkg); count--; } }
	void consumeItem(const ITEM& pkg, const Tagcoll::OpSet<Tag>& tags)
	{
		if (count > 0) { this->consumer->consume(pkg, tags); count--; }
	}

public:
	TFLimit(int count) : count(count) {}
	TFLimit(int count, Tagcoll::Consumer<ITEM, Tag>& cons)
		: Tagcoll::Filter<ITEM, Tag>(cons), count(count) {}
};

void* DebtagsDocument::SpecialsGen::main() throw ()
{
	time_t start = time(NULL);
	fprintf(stderr, "SpecialsGen thread start\n");
	{
		MutexLock lock(doc.specialsMutex);
		doc.hasSpecials = GENERATING;
	}

	doc.specials.clear();
	fprintf(stderr, "SpecialsGen thread: specials cleared\n");
	if (_interrupt) return 0;
	if (0)
	{
		// Limit the number of packages, to have decent times during debugging
		TFLimit<package> limFilter(100, &doc.specials);
		coll.output(limFilter);
	} else {
		coll.output(doc.specials);
	}
	fprintf(stderr, "SpecialsGen thread: specials fed data\n");
	if (_interrupt) return 0;
	doc.specials.compute(this);
	fprintf(stderr, "SpecialsGen thread: specials computed\n");

	{
		MutexLock lock(doc.specialsMutex);
		if (doc.specials.isComputed())
			doc.hasSpecials = YES;
		else
		{
			doc.specials.clear();
			doc.hasSpecials = NO;
		}
	}
	int elapsed = time(NULL) - start;
	fprintf(stderr, "SpecialsGen thread end (%d seconds)\n", elapsed);
	return 0;
}

void DebtagsDocument::regenerateSpecials() throw ()
{
	return;
	{
		MutexLock lock(specialsMutex);
		if (hasSpecials == GENERATING)
		{
			specialsGen.interrupt();
			specialsMutex.unlock();
			specialsGen.join();
			specialsMutex.lock();
		}
		hasSpecials = GENERATING;
	}
	specialsGen.interrupt(false);
	specialsGen.coll = Tagcoll::ItemGrouper<aptFront::cache::entity::Package, Tag>();
	tagdb().output(specialsGen.coll);
	specialsGen.start();
}

void DebtagsDocument::wantSpecials() throw ()
{
	if (hasSpecials == NO)
	{
		specialsMutex.unlock();
		regenerateSpecials();
		specialsMutex.lock();
	}

	if (hasSpecials == GENERATING)
	{
		specialsMutex.unlock();
		specialsGen.join();
		specialsMutex.lock();
	}
}
#endif

void DebtagsDocument::do_changed()
{
	fprintf(stderr, "changed\n");
	if (computeIntensive)
	{
#if 0
		regenerateSpecials();
#endif
	}

	_signal_changed.emit();
}

void DebtagsDocument::do_reselected()
{
	fprintf(stderr, "reselected\n");

	_signal_reselected.emit();
}

bool DebtagsDocument::canUndo() const
{
	return _undoTail > 0;
}

bool DebtagsDocument::canRedo() const
{
	return _undoTail < _undoBuffer.size();
}

bool DebtagsDocument::canReapply(const Package& pkg) const
{
	return _lastPatch.getItem() != Package() &&
		   _lastPatch.getItem() != pkg;
}

void DebtagsDocument::undo()
{
	if (!canUndo())
		return;

	debug("Pre undo: tail: %d, size: %d\n", _undoTail, _undoBuffer.size());

	_undoTail--;
	debtags().tagdb().applyChange(_undoBuffer[_undoTail]);
	_undoBuffer[_undoTail] = _undoBuffer[_undoTail].getReverse();

	debug("Post undo: tail: %d, size: %d\n", _undoTail, _undoBuffer.size());

	do_changed();
}

void DebtagsDocument::redo()
{
	if (!canRedo())
		return;

	debug("Pre redo: tail: %d, size: %d\n", _undoTail, _undoBuffer.size());

	debtags().tagdb().applyChange(_undoBuffer[_undoTail]);
	_undoBuffer[_undoTail] = _undoBuffer[_undoTail].getReverse();
	_undoTail++;


	debug("Post redo: tail: %d, size: %d\n", _undoTail, _undoBuffer.size());

	do_changed();
}

void DebtagsDocument::reapply(const Package& pkg)
{
	if (!canReapply(pkg))
		return;

	// Build the new patch
	Tagcoll::Patch<Package, Tag> newPatch(
			pkg,
			_lastPatch.getAdded(),
			_lastPatch.getRemoved());

	applyChange(newPatch);
}

void DebtagsDocument::setCurrent(const Package& pkg)
{
	if (_current != pkg)
	{
		_current = pkg;
		do_reselected();
	}
}

void DebtagsDocument::applyChange(const Patch<Package, Tag>& change)
{
	PatchList<Package, Tag> patchList;
	patchList.addPatch(change);
	applyChange(patchList);
}

void DebtagsDocument::applyChange(const PatchList<Package, Tag>& change)
{
	debtags().tagdb().applyChange(change);
	
	if (1 /*reverse != change*/)
	{
		debug("Pre change: tail: %d, size: %d\n", _undoTail, _undoBuffer.size());

		// Push the reverse change in the undo buffer
		if (_undoTail == _undoBuffer.size())
		{
			_undoBuffer.push_back(change.getReverse());
			_undoTail++;
		}
		else
		{
			_undoBuffer[_undoTail++] = change.getReverse();
			_undoBuffer.resize(_undoTail);
		}

		// Save 'last patch' if it's only about one package
		if (change.size() == 1)
			_lastPatch = change.begin()->second;
		else
			_lastPatch = Tagcoll::Patch<Package, Tag>(Package());

		debug("Post change: tail: %d, size: %d\n", _undoTail, _undoBuffer.size());

		_changed = true;
			
		do_changed();
	} else
		debug("Change had no effect\n");
}

/*
template<class T>
void DebtagsDocument<T>::load()
	throw (FileException, ParserException)
{
	//_collection.clear();
	DebtagsEnvironment::get().outputPatched(_collection);

	// Output the merged collection to consumer
	/ *
	TagcollBuilder builder(_handleMaker);
	merger.output(builder);
	_collection = builder.collection();
	* /
	_undoBuffer.clear();
	_undoTail = 0;
	_fileName = "";

	do_filename_changed();
	do_changed();
}
*/

void DebtagsDocument::save() throw (FileException)
{
	debtags().savePatch();
	_changed = false;
	//DebtagsEnvironment::get().savePatch(_collection);
	/*
	FILE* out = fopen(file.c_str(), "wt");
	if (!out)
		throw FileException(errno, "opening file " + file);
	
	// Output the collection, grouping the items
	TagcollSerializer serializer(out);
	ItemGrouper<string> grouper;
	output(grouper);
	grouper.output(serializer);

	fclose(out);

	if (file != _fileName)
	{
		_fileName = file;
		do_filename_changed();
	}
	*/
}

void DebtagsDocument::send() throw (FileException)
{
	debtags().sendPatch();
}

/*
template<class T>
OpSet<T> DebtagsDocument<T>::handlesToItems(const OpSet<int>& handles) const throw ()
{
	OpSet<T> res;
	for (set<int>::const_iterator i = handles.begin(); i != handles.end(); i++)
		res += _handleMaker.getItem(*i);
	return res;
}
*/

/*
template<class T>
void DebtagsDocument<T>::output(TagcollConsumer<T>& consumer) throw ()
{
	_collection.output(consumer);
	/ *
	for (typename TagCollection<int>::const_iterator i = _collection.begin();
			i != _collection.end(); i++)
		consumer.consume(handlesToItems(i->second), i->first);
	* /
}*/

/*
template<class T>
void DebtagsDocument<T>::output(TagcollConsumer<int>& consumer) throw ()
{
	for (typename TagCollection<int>::const_iterator i = _collection.begin();
			i != _collection.end(); i++)
		consumer.consume(i->second, i->first);
}*/
// vim:set ts=4 sw=4:
