#ifndef DEBTAGS_DOCUMENT_H
#define DEBTAGS_DOCUMENT_H

/*
 * Tagged Collection document for a Document-View editing architecture
 *
 * Copyright (C) 2003  Enrico Zini <enrico@debian.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <sigc++/sigc++.h>

#include <apt-front/cache/entity/package.h>
#include <apt-front/cache/entity/tag.h>
#include <apt-front/cache/component/packagetags.h>
#include <apt-front/cache/component/tags.h>

#include <tagcoll/Patches.h>
#include <tagcoll/InputMerger.h>
#include <tagcoll/ItemGrouper.h>

//#include <debtags/Utils.h>

#include "Mutex.h"
#include "Thread.h"


#include <vector>

class DebtagsDocument // : public Debtags::DebtagsSimple<aptFront::cache::entity::Package>
{
public:
	typedef SigC::Signal0<void> type_signal_changed;
	typedef SigC::Signal0<void> type_signal_reselected;
	typedef SigC::Signal0<void> type_signal_filename_changed;

	typedef aptFront::cache::entity::Package Package;
	typedef aptFront::cache::entity::Facet Facet;
	typedef aptFront::cache::entity::Tag Tag;

protected:
#if 0
	class SpecialsGen : public Thread, public Debtags::Status
	{
	protected:
		DebtagsDocument& doc;
		Tagcoll::ItemGrouper<package, Tag> coll;
		bool _interrupt;

		virtual const char* threadTag() { return "SpecialsGen"; }

		virtual void* main() throw ();

	public:
		SpecialsGen(DebtagsDocument& doc) : doc(doc), _interrupt(false) {}
		virtual ~SpecialsGen() throw () {}

		void interrupt(bool val = true) throw () { _interrupt = val; }
		bool advance(int steps = 1) throw ();
	
		friend class DebtagsDocument;
	};
#endif

	aptFront::cache::component::Packages& _packages;
	aptFront::cache::component::PackageTags& _debtags;
	aptFront::cache::component::Tags& _voc;
	
	bool _changed;

	Package _current;

	Tagcoll::Patch<Package, Tag> _lastPatch;
	std::vector<Tagcoll::PatchList<Package, Tag> > _undoBuffer;
	unsigned int _undoTail;

	type_signal_changed _signal_changed;
	type_signal_reselected _signal_reselected;
	type_signal_filename_changed _signal_filename_changed;

	//OpSet<T> handlesToItems(const OpSet<int>& handles) const throw ();
	
#if 0
	SpecialsGen specialsGen;
#endif

	bool computeIntensive;
	
public:
	DebtagsDocument() throw ();
	virtual ~DebtagsDocument();

	aptFront::cache::component::Packages& packages() { return _packages; }
	const aptFront::cache::component::Packages& packages() const { return _packages; }

	aptFront::cache::component::PackageTags& debtags() { return _debtags; }
	const aptFront::cache::component::PackageTags& debtags() const { return _debtags; }

	aptFront::cache::component::Tags& vocabulary() { return _voc; }
	const aptFront::cache::component::Tags& vocabulary() const { return _voc; }

	const Tagcoll::PatchList<Package, Tag>& getPatch() const
	{
		return debtags().tagdb().getChanges();
	}

	bool getComputeIntensive() const throw () { return computeIntensive; }
	void setComputeIntensive(bool val) throw ()
	{
		if (!computeIntensive && val)
		{
			computeIntensive = val;
#if 0
			//regenerateSpecials();
#endif
		} else
			computeIntensive = val;
	}
	
	//HandleMaker<T>& handles() throw () { return _handleMaker; }
	//Debtags::Packages& collection() throw () { return _collection; }
	//const Debtags::Packages& collection() const throw () { return _collection; }

	bool changed() const { return _changed; }

	Package& current() { return _current; }
	const Package& current() const { return _current; }

	Tagcoll::OpSet<Tag> currentTags() const
	{
		if (_current.valid())
			return debtags().tagdb().getTags(_current);
		else
			return Tagcoll::OpSet<Tag>();
	}

	void setCurrent(const Package& pkg);

	// Apply a change, appending the reverse change in the undo buffer
	void applyChange(const Tagcoll::Patch<Package, Tag>& change);
	void applyChange(const Tagcoll::PatchList<Package, Tag>& change);

	bool canUndo() const;
	bool canRedo() const;
	bool canReapply(const Package& pkg) const;

	void undo();
	void redo();
	void reapplyToCurrent() { reapply(current()); }
	void reapply() { reapply(current()); }
	void reapply(const Package& pkg);

	//void load() throw (FileException, ParserException);
	void save() throw (Tagcoll::FileException);
	void send() throw (Tagcoll::FileException);

	//void output(TagcollConsumer<T>& consumer) throw ();
	//void output(TagcollConsumer<int>& consumer) throw ();

	// Results of the package filter
	//Debtags::PackageSet currentPackages;
	Tagcoll::InputMerger<Package, Tag> subCollection;

	//Debtags::Specials<Package> specials;
	enum { YES, NO, GENERATING } hasSpecials;
	Mutex specialsMutex;

	/**
	 * Fork a subthread to regenerate the specials.
	 *
	 * If a previous subthread is stull running, it is interrupted and restarted.
	 */
#if 0
	void regenerateSpecials() throw ();

	/**
	 * Request the specials to be fully generated
	 *
	 * Must be called when specialsMutex is locked.  On exit, the specials are
	 * guaranteed to have been generated.
	 */
	void wantSpecials() throw ();
#endif

	type_signal_changed signal_changed() throw () { return _signal_changed; }
	type_signal_reselected signal_reselected() throw () { return _signal_reselected; }
	type_signal_filename_changed signal_filename_changed() throw () { return _signal_filename_changed; }

	virtual void do_changed();
	virtual void do_reselected();

	friend class SpecialsGen;
};

// vim:set ts=4 sw=4:
#endif
